class ArrayHandler(object):

    def __init__(self, group_lims=None):
        self.group_lims = group_lims

    def aggregate(self, array, aggr_level):
        if aggr_level < 1:
            for elm in array:
                yield elm
        if self.group_lims is None:
            self.group_lims = [list() for k in range(aggr_level)]
            i = -1
        else:
            try:
                i = self.group_lims[aggr_level - 1][-1]
            except IndexError:
                i = -1
        if aggr_level == 1:
            for array_elm in array:
                for elm in array_elm:
                    yield elm
                i += len(array_elm)
                self.group_lims[0].append(i)
        else:
            for array_elm in array:
                for elm in self.aggregate(array=array_elm, aggr_level=aggr_level-1):
                    yield elm
                i += len(array_elm)
                self.group_lims[aggr_level-1].append(i)


def group_array(aggr_array, group_lims):
    if len(group_lims) == 1:
        return _group_level(aggr_array, group_lims.pop(0))
    aggr_array = _group_level(aggr_array, group_lims.pop(0))
    return group_array(aggr_array, group_lims)


def _group_level(aggr_array, level_lims):
    grouped_array = [list()]
    for i in range(len(aggr_array)):
        grouped_array[-1].append(aggr_array[i])
        if i == level_lims[0]:
            level_lims.pop(0)
            grouped_array.append(list())
    return list(filter(None, grouped_array))



from functools import reduce
from operator import getitem

from jsondler import JsonEntry


class ModelScheme(JsonEntry):

    method_name_key = "method_name"
    model_path_key = "model_path"
    basic_models_key = "basic_models"

    method_name_0 = None
    model_path_0 = None
    basic_models_0 = dict()

    def __init__(self, method_name=method_name_0, model_path=model_path_0, basic_models=None):
        if basic_models is None:
            basic_models = self.basic_models_0

        self.method_name = method_name
        self.model_path = model_path
        self.basic_models = basic_models

    @classmethod
    def attr_scheme(cls):
        attr_scheme={
            'method_name': (cls.method_name_key,),
            'model_path': (cls.model_path_key,),
            'basic_models': (cls.basic_models_key,),
        }
        return attr_scheme

    def load_from_dict(cls, in_dict):
        model_scheme = super(ModelScheme, cls).load_from_dict(in_dict=in_dict)
        for basic_model_key in model_scheme.basic_models:
            model_scheme.basic_models[basic_model_key] = \
                cls.load_from_dict(in_dict=model_scheme.basic_models[basic_model_key])
        return model_scheme

    def get_json(self):
        got_json = super(ModelScheme, self).get_json()
        for basic_model_key in reduce(getitem, self.attr_scheme()["basic_models"], got_json):
            reduce(getitem, self.attr_scheme()["basic_models"], got_json)[basic_model_key] = \
                reduce(getitem, self.attr_scheme()["basic_models"], got_json)[basic_model_key].get_json()
        return got_json
